@@ -1,16 +1,28 @@
+import sys
 import threading
+import weakref
+from base64 import b64encode
+from hashlib import sha1
 from logging import getLogger
 from os import urandom
-from hashlib import sha1
-import weakref
 
 from redis import StrictRedis
 from redis.exceptions import NoScriptError
 
-__version__ = "3.1.0"
+__version__ = '3.5.0'
 
 logger = getLogger(__name__)
 
+PY3 = sys.version_info[0] == 3
+
+if PY3:
+    text_type = str
+    binary_type = bytes
+else:
+    text_type = unicode  # noqa
+    binary_type = str
+
+
 # Check if the id match. If not, return an error code.
 UNLOCK_SCRIPT = b"""
     if redis.call("get", KEYS[1]) ~= ARGV[1] then
@@ -18,7 +30,7 @@ UNLOCK_SCRIPT = b"""
     else
         redis.call("del", KEYS[2])
         redis.call("lpush", KEYS[2], 1)
-        redis.call("expire", KEYS[2], 1)
+        redis.call("pexpire", KEYS[2], KEYS[3])
         redis.call("del", KEYS[1])
         return 0
     end
@@ -41,7 +53,7 @@ EXTEND_SCRIPT_HASH = sha1(EXTEND_SCRIPT).hexdigest()
 RESET_SCRIPT = b"""
     redis.call('del', KEYS[2])
     redis.call('lpush', KEYS[2], 1)
-    redis.call('expire', KEYS[2], 1)
+    redis.call('pexpire', KEYS[2], KEYS[3])
     return redis.call('del', KEYS[1])
 """
 
@@ -91,10 +103,10 @@ class NotExpirable(RuntimeError):
     pass
 
 
-((UNLOCK, _, _,   # noqa
-  EXTEND, _, _,
-  RESET, _, _,
-  RESET_ALL, _, _),
+((UNLOCK_SCRIPT, _, _,  # noqa
+  EXTEND_SCRIPT, _, _,
+  RESET_SCRIPT, _, _,
+  RESET_ALL_SCRIPT, _, _),
  SCRIPTS) = zip(*enumerate([
     UNLOCK_SCRIPT_HASH, UNLOCK_SCRIPT, 'UNLOCK_SCRIPT',
     EXTEND_SCRIPT_HASH, EXTEND_SCRIPT, 'EXTEND_SCRIPT',
@@ -113,7 +125,7 @@ def _eval_script(redis, script_id, *keys, **kwargs):
     try:
         return redis.evalsha(SCRIPTS[script_id], len(keys), *keys + args)
     except NoScriptError:
-        logger.warn("%s not cached.", SCRIPTS[script_id + 2])
+        logger.info("%s not cached.", SCRIPTS[script_id + 2])
         return redis.eval(SCRIPTS[script_id + 1], len(keys), *keys + args)
 
 
@@ -122,7 +134,7 @@ class Lock(object):
     A Lock context manager implemented via redis SETNX/BLPOP.
     """
 
-    def __init__(self, redis_client, name, expire=None, id=None, auto_renewal=False):
+    def __init__(self, redis_client, name, expire=None, id=None, auto_renewal=False, strict=True, signal_expire=1000):
         """
         :param redis_client:
             An instance of :class:`~StrictRedis`.
@@ -138,23 +150,39 @@ class Lock(object):
             Note that if you specify this then the lock is marked as "held". Acquires
             won't be possible.
         :param auto_renewal:
-            If set to True, Lock will automatically renew the lock so that it
+            If set to ``True``, Lock will automatically renew the lock so that it
             doesn't expire for as long as the lock is held (acquire() called
             or running in a context manager).
 
             Implementation note: Renewal will happen using a daemon thread with
-            an interval of expire*2/3. If wishing to use a different renewal
-            time, subclass Lock, call super().__init__() then set
-            self._lock_renewal_interval to your desired interval.
+            an interval of ``expire*2/3``. If wishing to use a different renewal
+            time, subclass Lock, call ``super().__init__()`` then set
+            ``self._lock_renewal_interval`` to your desired interval.
+        :param strict:
+            If set ``True`` then the ``redis_client`` needs to be an instance of ``redis.StrictRedis``.
+        :param signal_expire:
+            Advanced option to override signal list expiration in milliseconds. Increase it for very slow clients. Default: ``1000``.
         """
-        assert isinstance(redis_client, StrictRedis)
+        if strict and not isinstance(redis_client, StrictRedis):
+            raise ValueError("redis_client must be instance of StrictRedis. "
+                             "Use strict=False if you know what you're doing.")
         if auto_renewal and expire is None:
             raise ValueError("Expire may not be None when auto_renewal is set")
 
         self._client = redis_client
         self._expire = expire if expire is None else int(expire)
-        self._id = urandom(16) if id is None else id
-        self._held = id is not None
+        self._signal_expire = signal_expire
+        if id is None:
+            self._id = b64encode(urandom(18)).decode('ascii')
+        elif isinstance(id, binary_type):
+            try:
+                self._id = id.decode('ascii')
+            except UnicodeDecodeError:
+                self._id = b64encode(id).decode('ascii')
+        elif isinstance(id, text_type):
+            self._id = id
+        else:
+            raise TypeError("Incorrect type for `id`. Must be bytes/str not %s." % type(id))
         self._name = 'lock:'+name
         self._signal = 'lock-signal:'+name
         self._lock_renewal_interval = (float(expire)*2/3
@@ -162,18 +190,25 @@ class Lock(object):
                                        else None)
         self._lock_renewal_thread = None
 
+    @property
+    def _held(self):
+        return self.id == self.get_owner_id()
+
     def reset(self):
         """
         Forcibly deletes the lock. Use this with care.
         """
-        _eval_script(self._client, RESET, self._name, self._signal)
+        _eval_script(self._client, RESET_SCRIPT, self._name, self._signal, self._signal_expire)
 
     @property
     def id(self):
         return self._id
 
     def get_owner_id(self):
-        return self._client.get(self._name)
+        owner_id = self._client.get(self._name)
+        if isinstance(owner_id, binary_type):
+            owner_id = owner_id.decode('ascii', 'replace')
+        return owner_id
 
     def acquire(self, blocking=True, timeout=None):
         """
@@ -212,7 +247,6 @@ class Lock(object):
                     return False
 
         logger.debug("Got lock for %r.", self._name)
-        self._held = True
         if self._lock_renewal_interval is not None:
             self._start_lock_renewer()
         return True
@@ -232,7 +266,7 @@ class Lock(object):
                     "To extend a lock 'expire' must be provided as an "
                     "argument to extend() method or at initialization time."
                 )
-        error = _eval_script(self._client, EXTEND, self._name, args=(expire, self._id))
+        error = _eval_script(self._client, EXTEND_SCRIPT, self._name, args=(expire, self._id))
         if error == 1:
             raise NotAcquired("Lock %s is not acquired or it already expired." % self._name)
         elif error == 2:
@@ -313,22 +347,27 @@ class Lock(object):
             * Use ``Lock("name", id=id_from_other_place).release()``
             * Use ``Lock("name").reset()``
         """
-        if not self._held:
-            raise NotAcquired("This Lock instance didn't acquire the lock.")
         if self._lock_renewal_thread is not None:
             self._stop_lock_renewer()
         logger.debug("Releasing %r.", self._name)
-        error = _eval_script(self._client, UNLOCK, self._name, self._signal, args=(self._id,))
+        error = _eval_script(self._client, UNLOCK_SCRIPT, self._name, self._signal, self._signal_expire, args=(self._id,))
         if error == 1:
             raise NotAcquired("Lock %s is not acquired or it already expired." % self._name)
         elif error:
             raise RuntimeError("Unsupported error code %s from EXTEND script." % error)
-        else:
-            self._held = False
+
+    def locked(self):
+        """
+        Return true if the lock is acquired.
+
+        Checks that lock with same name already exists. This method returns true, even if
+        lock have another id.
+        """
+        return self._client.exists(self._name) == 1
 
 
 def reset_all(redis_client):
     """
     Forcibly deletes all locks if its remains (like a crash reason). Use this with care.
     """
-    _eval_script(redis_client, RESET_ALL)
+    _eval_script(redis_client, RESET_ALL_SCRIPT)
